// X and Y are direct drive...
// 16 * 200 steps per revolution, 10 teeth, 5.08mm per tooth
// 16 * 200 / 10 / 5.08 = 62.992126 steps per mm
#define STEPS_PER_MM_X 62.992126
#define STEPS_PER_MM_Y 62.992126

// Z is geared...
// drive gear: 16 * 200 steps per revolution, 10 teeth
// driven gear: 25 teeth, 1.25 mm per revolution (M8 pitch)
// 16 * 200 / 10 * 25 / 1.25 = 6400 steps per mm
#define STEPS_PER_MM_Z 6400

// E...
#define STEPS_PER_CUBIC_MM_E 190

// extruder advance constant (s2/mm3)
//
// advance (steps) = STEPS_PER_CUBIC_MM_E * EXTUDER_ADVANCE_K * cubic mm per second ^ 2
//
// model the extruder as a spring followed by an incompressible fluid...
// hooke's law says:            force = k * distance
// bernoulli's priniciple says: v ^ 2 / 2 + g . h + pressure / density = constant
// ...so: v ^ 2 is proportional to number of steps we 'advance' the extruder
#define EXTRUDER_ADVANCE_K 0.010

// default extrusion area for M101
#define INIT_EXTRUSION_AREA (0.465 * 0.31)

// feedrates (mm/min)
#define INIT_FEEDRATE 4000
#define HOME_FEEDRATE_XY 4000
#define HOME_FEEDRATE_Z 150
#define MIN_FEEDRATE 50
#define MAX_FEEDRATE_XY 20000
#define MAX_FEEDRATE_Z 150

// acceleration (mm/min/s)
#define ACCELERATION_XY 50000
#define ACCELERATION_Z 10000

// steppers to disable after every move
#define AUTO_DISABLE_X 0
#define AUTO_DISABLE_Y 0
#define AUTO_DISABLE_Z 1
#define AUTO_DISABLE_E 0

// stepper directions
#define INVERT_X 1
#define INVERT_Y 1
#define INVERT_Z 0
#define INVERT_E 0

// minimum stepper pulse widths (us)
#define STEP_HIGH_US 1
#define STEP_LOW_US 1

// thermistor
//     +ve --- R2 -+- THERMISTOR -+- gnd
//                 |              |
//     adc --------+----- R1 -----+
#define THERMISTOR_R0_OHMS 100000
#define THERMISTOR_T0_DEG_C 25
#define THERMISTOR_BETA 4036
#define THERMISTOR_R1_OHMS 0      // 0 means missing
#define THERMISTOR_R2_OHMS 4500

// move buffer size
#define DDA_BUFFER 8

// comms
#define UART_BAUD 38400
#define UART_BUFFER 4

// dda performance tradeoffs
#define DDA_USE_APPROX_DIVIDE
#define DDA_SPEEDCHANGE_RATE 4  // recalculate speed every xth tick
#define DDA_SHIFT 20            // defines how accuracy decreases with feedrate, higher numbers are more accurate (within reason)
