#include <stdio.h>
#include <string.h>
#include <compat/console.h>
#include <compat/err.h>
#include <compat/serial.h>
#include "config.h"

void waitfor(serial_device_t device, char *s) {
    int c, i = 0;

    while (s[i] && !console_quit) {
        c = serial_getc(device, 1);
        if (c != SERIAL_ERROR && c != SERIAL_TIMEOUT) {
            if (c != s[i]) {
                for (int j = 0; j < i; j++)
                    console_putc(s[j]);
                console_putc(c);
                i = 0;
            }
            if (c == s[i])
                i++;
        }
    }
}

void sendfile(serial_device_t device, FILE *image) {
    int c, incomment, nl, ignore;

    incomment = 0;
    nl = 1;
    while(!console_quit) {
        c = fgetc(image);
        ignore = 0;

        switch(c) {
        case EOF:
            if (!nl) {
                serial_putc(device, '\n');
                console_putc('\n');
            }
            return;

        case '(':
            incomment = 1;
            break;

        case ' ':
        case '\n':
        case '\r':
            if (nl)
                ignore = 1;
        }

        if (!incomment && !ignore) {
            nl = 0;
            serial_putc(device, c);
            console_putc(c);
            if (c == '\n') {
                waitfor(device, "ok");
                waitfor(device, "\n");
                nl = 1;
            }
        }

        if (c == ')')
            incomment = 0;
    }
}

void terminal(serial_device_t device) {
    int c;

    while (!console_quit) {
        for(;;) {
            c = console_getc(1);
            if (c == CONSOLE_TIMEOUT)
                break;
            serial_putc(device, c);
            console_putc(c);
        }
        for(;;) {
            c = serial_getc(device, 1);
            if (c == SERIAL_TIMEOUT)
                break;
            if (c == SERIAL_ERROR)
                return;
            console_putc(c);
        }
    }
}

int main(int argc, char **argv) {
    char *progname = argv[0];
    serial_device_t device;
    FILE *image;

    if (argc == 3) {
        image = fopen(argv[2], "rb");
        if (image == NULL)
            err(1, "cannot open %s", argv[2]);
    } else
        image = NULL;

    if (argc < 2 || argc > 3) {
        fprintf(stderr, "usage: %s device [file.gcode]\n", progname);
        return 1;
    }

    device = serial_opendevice(argv[1], BAUD(UART_BAUD));
    console_init();
    if (image)
        sendfile(device, image);
    terminal(device);
    console_fini();
    printf("\n");
    return 0;
}
