#include <libavr/bits.h>
#include <libavr/interrupt.h>
#include <stdint.h>
#include "hw.h"
#include "../delay.h"

#define SPEEDCHANGE_HZ 8000

void delay_init(void) {
    /* use timer 1 for delay_ticked() (/64) */
    TCCR1A = 0<<WGM11 | 0<<WGM10;
    TCCR1B = 0<<WGM13 | 1<<WGM12 | 0<<CS12 | 1<<CS11 | 1<<CS10;
    OCR1A = 0;
    TCNT1 = 0;
}

void delay_tick_reset(void) {
    TCNT1 = 0;
    OCR1A = 0xFFFF;
    sbi(TIFR1, OCF1A);
}

void delay_tick_in(uint16_t us) {
#if F_CPU == 16000000UL
    us >>= 2;
#elif F_CPU == 8000000UL
    us >>= 3;
#else
    us = ((uint32_t)us) * (F_CPU / 62500) / 1024; // 62500 = 64 * 1e6 / 1024
#endif
    OCR1A = us;
    if (TCNT1 > us)
        TCNT1 = us - 1;
}

uint8_t delay_ticked(void) {
    if (bit_is_clear(TIFR1, OCF1A))
        return 0;
    sbi(TIFR1, OCF1A);
    return 1;
}
