#include <libavr/bits.h>
#include <libavr/device.h>
#include <libavr/flash.h>
#include <libavr/interrupt.h>
#include <libavr/uart.h>
#include <stdint.h>
#include "hw.h"
#include "../cmd.h"
#include "../config.h"
#include "../dda.h"
#include "../delay.h"
#include "../fan.h"
#include "../heater.h"
#include "../motor.h"
#include "../parse.h"
#include "../uart.h"
#include "../valve.h"

#if UART_BUFFER == 1 || UART_BUFFER == 4 || UART_BUFFER == 8 || UART_BUFFER == 16 || UART_BUFFER == 32 || UART_BUFFER == 64 || UART_BUFFER == 128 || UART_BUFFER == 256
#else
#error UART_BUFFER must be 1, 4, 8, 16, 32, 64, 128 or 256
#endif

#if UART_BUFFER == 1
uint8_t uart_getc(void) {
    while (bit_is_clear(UCSR0A, RXC0))
        dda_tick();
    return UDR0;
}

#else
static volatile uint8_t uart_getc_buf[UART_BUFFER];
static volatile uint8_t uart_getc_buf_t = 1;
static volatile uint8_t uart_getc_buf_h = 0;

SIGNAL(USART0_RX) {
    uart_getc_buf[uart_getc_buf_t] = UDR0;
    uart_getc_buf_t = (uart_getc_buf_t + 1) & (UART_BUFFER - 1);
}

uint8_t uart_getc(void) {
    uart_getc_buf_h = (uart_getc_buf_h + 1) & (UART_BUFFER - 1);
    while (uart_getc_buf_h == uart_getc_buf_t)
        dda_tick();
    return uart_getc_buf[uart_getc_buf_h];
}
#endif

void uart_putc(uint8_t c) {
    UDR0 = c;
    while (bit_is_clear(UCSR0A, UDRE0))
#ifdef DEBUG
        /* nothing */;
#else
        dda_tick();
#endif
    sbi(UCSR0A, UDRE0);
}

void uart_putsf(faddr_t s) {
    for(;;) {
        uint8_t c = flash_read(s++);
        if (c)
            uart_putc(c);
        else
            break;
    }
}

void main(void) {
    fan_init();
    heater_init();
    motor_init();
    delay_init();
    valve_init();
    UART_INIT(0, BAUD(UART_BAUD), 'N', 8, 1);
#if UART_BUFFER > 1
    sbi(UCSR0B, RXCIE0);
#endif
    sei();
    uart_puts("start\n");

    for(;;) {
        parse_cmd();
        cmd_process();
    }
}
