#include <libavr/bits.h>
#include <libavr/delay.h>
#include <stdint.h>
#include "hw.h"
#include "../config.h"
#include "../motor.h"
#include "../state.h"

#define AXIS(A) \
    SBI(DDR, ENABLE_##A); \
    motor_enable(A, !AUTO_DISABLE_##A); \
    SBI(DDR, DIR_##A); \
    SBI(DDR, STEP_##A);

void motor_init(void) { AXIS(X); AXIS(Y); AXIS(Z); AXIS(E); }
#undef AXIS

#define CSBI(x, A, B) do { if (x) SBI(A, B); else CBI(A, B); } while(0)

void motor_dir(int8_t a, int8_t d) {
    switch(a) {
    case X: CSBI(d ^ INVERT_X, PORT, DIR_X); break;
    case Y: CSBI(d ^ INVERT_Y, PORT, DIR_Y); break;
    case Z: CSBI(d ^ INVERT_Z, PORT, DIR_Z); break;
    case E: CSBI(d ^ INVERT_E, PORT, DIR_E); break;
    }
}

void motor_enable(int8_t a, int8_t e) {
    switch(a) {
    case X: CSBI(!e, PORT, ENABLE_X); break;
    case Y: CSBI(!e, PORT, ENABLE_Y); break;
    case Z: CSBI(!e, PORT, ENABLE_Z); break;
    case E: CSBI(!e, PORT, ENABLE_E); break;
    }
}

void motor_step(int8_t a) { \
    switch(a) {
    case X: SBI(PORT, STEP_X); break;
    case Y: SBI(PORT, STEP_Y); break;
    case Z: SBI(PORT, STEP_Z); break;
    case E: SBI(PORT, STEP_E); break;
    }
}

void motor_step_done(void) {
    delay_us(STEP_HIGH_US);
    CBI(PORT, STEP_X);
    CBI(PORT, STEP_Y);
    CBI(PORT, STEP_Z);
    CBI(PORT, STEP_E);
    delay_us(STEP_LOW_US);
}

void motor_steps(int8_t a, int16_t steps) {
    motor_dir(a, 1);
    if (steps < 0) {
        motor_dir(a, 0);
        steps = -steps;
    }
    while(steps--) {
        motor_step(a);
        motor_step_done();
    }
}
