#include <stdint.h>
#include "cmd.h"
#include "state.h"
#include "uart.h"

uint8_t parse_chk;
static uint8_t parse_seen_star, parse_c, parse_c_used = 1;

static void parse_ungetc() {
    parse_c_used = 0;
}

static uint8_t parse_getc() {
    if (parse_c_used) {
        if (parse_c == '\r') {
            parse_c = uart_getc();
            if (parse_c == '\n')
                parse_c = uart_getc();
        } else {
            parse_c = uart_getc();
        }
        if (parse_c == '*')
            parse_seen_star = 1;
        if (!parse_seen_star)
            parse_chk ^= parse_c;
    }
    parse_c_used = 1;
    if (parse_c == '\r')
        return '\n';
    return parse_c;
}

static int32_t parse_getn(int8_t decimal) {
    int32_t v;
    uint8_t c;
    int8_t state, sign;

    sign = parse_getc() == '-' ? -1 : 1;
    if (sign == 1)
        parse_ungetc();

    state = 0;
    v = 0;
    for (;;) {
        c = parse_getc();
        if (c >= '0' && c <= '9') {
            if (state < (PRECISION_DP+2)) {
                v = v * 10 + c - '0';
                if (state != 1)
                    state++;
            }
        } else if (c == '.' && state <= 1 && decimal) {
            state = 2;
        } else {
            parse_ungetc();
            break;
        }
    }

    if (state == 0)
        return UNSEEN;

    if (decimal) {
        if (state == 1)
            state = 2;
        for (; state < (PRECISION_DP+2); state++)
            v *= 10;
    }

    return sign * v;
}


void parse_cmd(void) {
    uint8_t i, c;

    for (i = 0; i < 27; i++)
        cmd[i] = UNSEEN;

    parse_chk = 0;
    parse_seen_star = 0;
    for(;;) {
        c = parse_getc();
        switch (c) {
        case ';':
            while (parse_getc() != '\n')
                /* nothing */;
            /* fallthough */
        case '\n':
            return;
        case '*':
            c = 26;
            break;
        default:
            c -= 'A';
            break;
        }

        if (c < 27) {
            // 0x03F7CFBF has bits clear for GMNT* and set for all other codes
            cmd[c] = parse_getn((0x03F7CFBFL >> c) & 1);
        }
    }
}
