#include <stdio.h>
#include <stdint.h>
#include <compat/err.h>
#include "cmd.h"
#include "dda.h"
#include "delay.h"
#include "fan.h"
#include "heater.h"
#include "motor.h"
#include "parse.h"
#include "state.h"
#include "uart.h"

#define QQ(X) #X
#define Q(X) QQ(X)
#define PRINT_DELAY if (show_hw && delay) { fprintf(stderr, ".%d.", delay); delay = 0; }

static int64_t us;
static int32_t delay;
static FILE *fp;
static int show_uart, show_hw, show_time;
static char axis[2][5] = {{'f', 'x', 'y', 'z', 'e'},
                          {'F', 'X', 'Y', 'Z', 'E'}};
static int8_t dir[5];


void cmd_print_i(char c, int i) {
    if (i != UNSEEN) printf("%c%d ", c, i);
}

void cmd_print_d(char c, int d) {
    if (d != UNSEEN) printf("%c%0." Q(PRECISION_DP) "f ", c, (double)d / PRECISION);
}

void cmd_print(void) {
    uint8_t c, i;
    uint32_t star;

    if (show_time)
        fprintf(stdout, "t=%0.6fs\t", (double)us/US_IN_S);
    for (i = 0; i < 4; i++) {
        c = "NGMT"[i];
        cmd_print_i(c, cmd[c - 'A']);
    }
    for (i = 0; i < 7; i++) {
        c = "XYZEFSP"[i];
        cmd_print_d(c, cmd[c - 'A']);
    }

    star = cmd[26];
    if (star != UNSEEN)
            printf("*%d ;%d", star, parse_chk);

    printf("\n");
}

void delay_ms(int x) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr,"delay_ms(%d)\n", x);
}
void delay_tick_reset(void) {
}
void delay_tick_in(uint16_t x) {
    delay += x;
    us += x;
}
uint8_t delay_ticked(void) {
    return 1;
}

void fan(int8_t f) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr,"fan(%d)\n", f);
}

int8_t heater_ready() {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr,"heater_ready()\n");
    return 1;
}
void heater_set_temp(uint8_t i, int16_t t) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr,"heater_set_temp(%d, %d)\n", i, t);
}
int16_t heater_get_temp(uint8_t i) {
    return i == BED ? 120 : 240;
}

void motor_dir(int8_t a, int8_t d) {
    dir[a] = d;
}
void motor_enable(int8_t a, int8_t d) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr, "motor_enable(%c,%d)\n", axis[0][a], d);
}
void motor_step(int8_t a) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr, "%c", axis[dir[a]][a]);
}
void motor_step_done(void) { }
void motor_steps(int8_t a, int16_t n) {
    motor_dir(a, n > 0);
    if (n < 0) n = -n;
    while(n--) motor_step(a);
}
uint8_t uart_getc(void) {
    return feof(fp) ? '\n' : fgetc(fp);
}
void uart_puts(char *s) {
    if (show_uart) printf("%s", s);
}
void uart_putc(uint8_t i) {
    if (show_uart) printf("%c", i);
}

void valve(uint8_t i, int8_t f) {
    PRINT_DELAY;
    if (show_hw) fprintf(stderr,"valve(%d,%d)\n", i, f);
}


int main(int argc, char **argv) {
    fp = stdin;
    show_uart = 1;
    show_time = 0;
    show_hw = 0;
    us = 0;
    delay = 0;

    while (argc >= 2 && argv[1][0] == '-') {
        switch (argv[1][1]) {
        case 'q':
            show_uart = 0;
            break;
        case 't':
            show_time = 1;
            break;
        case 'h':
            show_hw = 1;
            break;
        }
        argc--;
        argv++;
    }

    if (argc >= 2) {
        fp = fopen(argv[1], "r");
        if (fp == NULL)
            err(1, "cannot open %s", argv[1]);
    }


    while (!feof(fp)) {
        parse_cmd();
        cmd_print();
        cmd_process();
        dda_tick();
    }

    dda_drain();

    PRINT_DELAY;
    fprintf(stdout, "t=%0.6fs\n", (double)us/US_IN_S);
    fclose(fp);
    return 0;
}
